#!/usr/bin/env node

var fs = require("fs");
var ncp = require('ncp').ncp;
ncp.limit = 16;

var prompt = require('prompt');
prompt.start();

var schema = {
    properties: {
        name: {
            description: 'Name of the new laravel package',
        },
        namespace: {
            description: 'PHP Namespace for the new laravel package',
        },
        prefix: {
            description: 'PHP Classes Prefix',
        },
        viewsprefix:{
            description: 'Views package name (name::)',
        },
        folder: {
            description: 'Source folder (inside ./packages)',
        },
    }
};

prompt.get(schema, function (err, result) {

    /*
    * create the packages folder if it doesn't exist.
    * it is assumed that all package applications are
    * located inside the /packages folder
    * */
    if (!fs.existsSync(process.cwd() + '/packages')) {
        fs.mkdirSync(process.cwd() + '/packages');
    }
    var dir = process.cwd() + '/packages/'+result.folder;

    if (!fs.existsSync(dir)) {
        /*
            create the new package folder.
        */

        var fldrs = result.folder.split('/');
        var trail = '';
        for(var i = 0; i < fldrs.length; i++) {
            trail = trail+'/'+fldrs[i];
            if (!fs.existsSync(process.cwd() + '/packages' + trail)) {
                fs.mkdirSync(process.cwd() + '/packages' + trail);
            }
        }

        ncp(__dirname + '/lib/structure', process.cwd() + '/packages/' + result.folder, function (err) {
            if (err) {
                return console.error(err);
            }

            var walkSync = function(dir, filelist) {

                var fs = fs || require('fs'),
                    files = fs.readdirSync(dir);
                filelist = filelist || [];

                files.forEach(function(file) {
                    if (fs.statSync(dir + '/' + file).isDirectory()) {
                        filelist = walkSync(dir + '/' + file + '/', filelist);
                    } else {
                        fs.readFile(dir + '/' + file, 'utf8', function (err, data) {
                            if (err) throw err;
                            data = data.toString()
                                .replace("{{namespace}}", result.namespace)
                                .replace("{{prefix}}", result.prefix)
                                .replace("{{viewsprefix}}", result.viewsprefix);

                            fs.writeFile (dir + '/' + file, data, function(err) {
                                if (err) throw err;
                            });
                            let dest = dir + '/' + file;
                            switch(file) {
                                case 'ServiceProvider.php':
                                case 'Controller.php':
                                    dest = dir + '/' + result.prefix + file;
                                    break;
                            }
                            if(dest !== dir + '/' + file) {
                                fs.rename(dir + '/' + file, dest, function (err) {
                                    if (err) throw err;
                                    console.log('renamed complete');
                                });
                            }
                        });
                        filelist.push(file);
                    }
                });
                return filelist;
            };
            walkSync(process.cwd() + '/packages/' + result.folder);

            /*
            * update the composer.json file
            * with the newly created package
            * inside the psr-4 json object
            * */
            fs.readFile(process.cwd() + '/composer.json', function(err, data) {
                if(err) {
                    return console.log(err);
                }
                var compserc = JSON.parse(data.toString());
                compserc.autoload['psr-4'][result.namespace+"\\"] = "packages/"+result.folder+"/src";
                fs.writeFile(process.cwd()+"/composer.json", JSON.stringify(compserc, null, 4), function(err) {
                    if(err) {
                        return console.log(err);
                    }
                    console.log("Composer.json file updated...");
                });
            });

            // console.log(walkSync(process.cwd() + '/packages/' + result.folder));
            // console.log('done!');
        });

        console.log('Your package file structure and home route are set up.');
        console.log('Copy and paste the following line in the providers section of laravel\'s config/app.php file: ');
        console.log(result.namespace+'\\'+result.prefix+'ServiceProvider::class');
        console.log("Don't forget to run cmnd: composer update!");

        return;
    }
    console.log('folder already exists please choose another one');
});